@@ -13,6 +13,7 @@ use crate::layout::Orientation;
 use crate::object_tree::*;
 use crate::parser::{NodeOrToken, SyntaxNode};
 use core::cell::RefCell;
+use std::cell::Cell;
 use std::collections::HashMap;
 use std::rc::{Rc, Weak};
 
@@ -45,6 +46,7 @@ pub enum BuiltinFunction {
     ColorBrighter,
     ColorDarker,
     ImageSize,
+    ArrayLength,
     Rgb,
     ImplicitLayoutInfo(Orientation),
     RegisterCustomFontByPath,
@@ -125,6 +127,9 @@ impl BuiltinFunction {
                 }),
                 args: vec![Type::Image],
             },
+            BuiltinFunction::ArrayLength => {
+                Type::Function { return_type: Box::new(Type::Int32), args: vec![Type::Model] }
+            }
             BuiltinFunction::Rgb => Type::Function {
                 return_type: Box::new(Type::Color),
                 args: vec![Type::Int32, Type::Int32, Type::Int32, Type::Float32],
@@ -168,6 +173,7 @@ impl BuiltinFunction {
             BuiltinFunction::ImageSize => true,
             #[cfg(target_arch = "wasm32")]
             BuiltinFunction::ImageSize => false,
+            BuiltinFunction::ArrayLength => true,
             BuiltinFunction::Rgb => true,
             BuiltinFunction::ImplicitLayoutInfo(_) => false,
             BuiltinFunction::RegisterCustomFontByPath
@@ -648,7 +654,7 @@ impl Expression {
             Expression::PathElements { elements } => {
                 if let Path::Elements(elements) = elements {
                     for element in elements {
-                        element.bindings.values().for_each(|binding| visitor(binding))
+                        element.bindings.values().for_each(|binding| visitor(&binding.borrow()))
                     }
                 }
             }
@@ -730,7 +736,10 @@ impl Expression {
             Expression::PathElements { elements } => {
                 if let Path::Elements(elements) = elements {
                     for element in elements {
-                        element.bindings.values_mut().for_each(|binding| visitor(binding))
+                        element
+                            .bindings
+                            .values_mut()
+                            .for_each(|binding| visitor(&mut binding.borrow_mut()))
                     }
                 }
             }
@@ -804,7 +813,7 @@ impl Expression {
                 if let Path::Elements(elements) = elements {
                     elements
                         .iter()
-                        .all(|element| element.bindings.values().all(|v| v.is_constant()))
+                        .all(|element| element.bindings.values().all(|v| v.borrow().is_constant()))
                 } else {
                     true
                 }
@@ -1006,9 +1015,8 @@ impl Expression {
             Type::PhysicalLength => Expression::NumberLiteral(0., Unit::Phx),
             Type::LogicalLength => Expression::NumberLiteral(0., Unit::Px),
             Type::Percent => Expression::NumberLiteral(100., Unit::Percent),
-            // FIXME: Is that correct?
             Type::Image => Expression::ImageReference {
-                resource_ref: ImageReference::AbsolutePath(String::new()),
+                resource_ref: ImageReference::None,
                 source_location: None,
             },
             Type::Bool => Expression::BoolLiteral(false),
@@ -1041,13 +1049,7 @@ impl Expression {
     pub fn try_set_rw(&mut self) -> bool {
         match self {
             Expression::PropertyReference(nr) => {
-                nr.element()
-                    .borrow()
-                    .property_analysis
-                    .borrow_mut()
-                    .entry(nr.name().to_owned())
-                    .or_default()
-                    .is_set = true;
+                nr.mark_as_set();
                 true
             }
             Expression::StructFieldAccess { base, .. } => base.try_set_rw(),
@@ -1074,7 +1076,7 @@ pub struct BindingExpression {
     pub animation: Option<PropertyAnimation>,
 
     /// The analysis information. None before it is computed
-    pub analysis: RefCell<Option<BindingAnalysis>>,
+    pub analysis: Option<BindingAnalysis>,
 
     /// The properties this expression is aliased with using two way bindings
     pub two_way_bindings: Vec<NamedReference>,
@@ -1166,7 +1168,7 @@ impl Spanned for BindingExpression {
 #[derive(Default, Debug, Clone)]
 pub struct BindingAnalysis {
     /// true if that binding is part of a binding loop that already has been reported.
-    pub is_in_binding_loop: bool,
+    pub is_in_binding_loop: Cell<bool>,
 
     /// true if the binding is a constant value that can be set without creating a binding at runtime
     pub is_const: bool,
@@ -1208,6 +1210,7 @@ pub enum ImageReference {
     None,
     AbsolutePath(String),
     EmbeddedData { resource_id: usize, extension: String },
+    EmbeddedTexture { resource_id: usize },
 }
 
 /// Print the expression as a .60 code (not necessarily valid .60)
