@@ -12,9 +12,10 @@ LICENSE END */
 use crate::diagnostics::BuildDiagnostics;
 use crate::expression_tree::*;
 use crate::langtype::{PropertyLookupResult, Type};
-use crate::object_tree::ElementRc;
+use crate::object_tree::{Component, ElementRc};
 
-use std::rc::Rc;
+use std::cell::RefCell;
+use std::rc::{Rc, Weak};
 
 #[derive(Clone, Debug, Copy, Eq, PartialEq)]
 pub enum Orientation {
@@ -155,38 +156,48 @@ impl LayoutConstraints {
             fixed_height: false,
         };
         let mut apply_size_constraint =
-            |prop, binding: &BindingExpression, op: &mut Option<NamedReference>| {
+            |prop,
+             binding: &BindingExpression,
+             enclosing1: &Weak<Component>,
+             depth,
+             op: &mut Option<NamedReference>| {
                 if let Some(other_prop) = op {
-                    let old_priority = other_prop
-                        .element()
-                        .borrow()
-                        .bindings
-                        .get(other_prop.name())
-                        .map_or(0, |b| b.priority);
-                    if old_priority <= binding.priority {
-                        diag.push_error(
-                            format!("Cannot specify both '{}' and '{}'", prop, other_prop.name()),
-                            binding,
-                        )
-                    }
+                    find_binding(
+                        &other_prop.element(),
+                        other_prop.name(),
+                        |old, enclosing2, d2| {
+                            if Weak::ptr_eq(enclosing1, enclosing2)
+                                && old.priority.saturating_add(d2)
+                                    <= binding.priority.saturating_add(depth)
+                            {
+                                diag.push_error(
+                                    format!(
+                                        "Cannot specify both '{}' and '{}'",
+                                        prop,
+                                        other_prop.name()
+                                    ),
+                                    binding,
+                                )
+                            }
+                        },
+                    );
                 }
                 *op = Some(NamedReference::new(element, prop))
             };
-        let e = element.borrow();
-        if let Some(s) = e.bindings.get("height") {
+        find_binding(element, "height", |s, enclosing, depth| {
             constraints.fixed_height = true;
-            apply_size_constraint("height", s, &mut constraints.min_height);
-            apply_size_constraint("height", s, &mut constraints.max_height);
-        }
-        if let Some(s) = e.bindings.get("width") {
+            apply_size_constraint("height", s, enclosing, depth, &mut constraints.min_height);
+            apply_size_constraint("height", s, enclosing, depth, &mut constraints.max_height);
+        });
+        find_binding(element, "width", |s, enclosing, depth| {
             if s.expression.ty() == Type::Percent {
-                apply_size_constraint("width", s, &mut constraints.min_width);
+                apply_size_constraint("width", s, enclosing, depth, &mut constraints.min_width);
             } else {
                 constraints.fixed_width = true;
-                apply_size_constraint("width", s, &mut constraints.min_width);
-                apply_size_constraint("width", s, &mut constraints.max_width);
+                apply_size_constraint("width", s, enclosing, depth, &mut constraints.min_width);
+                apply_size_constraint("width", s, enclosing, depth, &mut constraints.max_width);
             }
-        }
+        });
 
         constraints
     }
@@ -334,35 +345,7 @@ impl LayoutGeometry {
         }
         self.padding.visit_named_references(visitor);
     }
-}
 
-/// Return a named reference to a property if a binding is set on that property
-fn binding_reference(element: &ElementRc, name: &str) -> Option<NamedReference> {
-    element.borrow().bindings.contains_key(name).then(|| NamedReference::new(element, name))
-}
-
-fn init_fake_property(
-    grid_layout_element: &ElementRc,
-    name: &str,
-    lazy_default: impl Fn() -> Option<NamedReference>,
-) {
-    if grid_layout_element.borrow().property_declarations.contains_key(name)
-        && !grid_layout_element.borrow().bindings.contains_key(name)
-    {
-        if let Some(e) = lazy_default() {
-            if e.name() == name && Rc::ptr_eq(&e.element(), grid_layout_element) {
-                // Don't reference self
-                return;
-            }
-            grid_layout_element
-                .borrow_mut()
-                .bindings
-                .insert(name.to_owned(), Expression::PropertyReference(e).into());
-        }
-    }
-}
-
-impl LayoutGeometry {
     pub fn new(layout_element: &ElementRc) -> Self {
         let spacing = binding_reference(layout_element, "spacing");
         let alignment = binding_reference(layout_element, "alignment");
@@ -386,6 +369,54 @@ impl LayoutGeometry {
     }
 }
 
+/// If this element or any of the parent has a binding to the property, call the functor with that binding, and the depth.
+/// Return None if the binding does not exist in any of the sub component, or Some with the result of the functor otherwise
+fn find_binding<R>(
+    element: &ElementRc,
+    name: &str,
+    f: impl FnOnce(&BindingExpression, &Weak<Component>, i32) -> R,
+) -> Option<R> {
+    let mut element = element.clone();
+    let mut depth = 0;
+    loop {
+        if let Some(b) = element.borrow().bindings.get(name) {
+            return Some(f(&b.borrow(), &element.borrow().enclosing_component, depth));
+        }
+        let e = match &element.borrow().base_type {
+            Type::Component(base) => base.root_element.clone(),
+            _ => return None,
+        };
+        element = e;
+        depth += 1;
+    }
+}
+
+/// Return a named reference to a property if a binding is set on that property
+fn binding_reference(element: &ElementRc, name: &str) -> Option<NamedReference> {
+    find_binding(element, name, |_, _, _| NamedReference::new(&element, name))
+}
+
+fn init_fake_property(
+    grid_layout_element: &ElementRc,
+    name: &str,
+    lazy_default: impl Fn() -> Option<NamedReference>,
+) {
+    if grid_layout_element.borrow().property_declarations.contains_key(name)
+        && !grid_layout_element.borrow().bindings.contains_key(name)
+    {
+        if let Some(e) = lazy_default() {
+            if e.name() == name && Rc::ptr_eq(&e.element(), grid_layout_element) {
+                // Don't reference self
+                return;
+            }
+            grid_layout_element
+                .borrow_mut()
+                .bindings
+                .insert(name.to_owned(), RefCell::new(Expression::PropertyReference(e).into()));
+        }
+    }
+}
+
 /// Internal representation of a grid layout
 #[derive(Debug, Clone)]
 pub struct GridLayout {
@@ -461,31 +492,49 @@ pub fn layout_info_type() -> Type {
 
 /// Get the implicit layout info of a particular element
 pub fn implicit_layout_info_call(elem: &ElementRc, orientation: Orientation) -> Expression {
-    if matches!(&elem.borrow().base_type ,
-                    Type::Builtin(base_type) if base_type.name == "Rectangle")
-    {
-        // hard-code the value for rectangle because many rectangle end up optimized away and we
-        // don't want to depend on the element.
-        Expression::Struct {
-            ty: layout_info_type(),
-            values: [("min", 0.), ("max", f32::MAX), ("preferred", 0.)]
-                .iter()
-                .map(|(s, v)| (s.to_string(), Expression::NumberLiteral(*v as _, Unit::Px)))
-                .chain(
-                    [("min_percent", 0.), ("max_percent", 100.), ("stretch", 1.)]
+    let mut elem_it = elem.clone();
+    loop {
+        return match &elem_it.clone().borrow().base_type {
+            Type::Component(base_comp) => {
+                match base_comp.root_element.borrow().layout_info_prop(orientation) {
+                    Some(nr) => {
+                        // We cannot take nr as is because it is relative to the elem's component. We therefore need to
+                        // use `elem` as an element for the PropertyReference, not `root` within the base of elem
+                        debug_assert!(Rc::ptr_eq(&nr.element(), &base_comp.root_element));
+                        Expression::PropertyReference(NamedReference::new(elem, nr.name()))
+                    }
+                    None => {
+                        elem_it = base_comp.root_element.clone();
+                        continue;
+                    }
+                }
+            }
+            Type::Builtin(base_type) if base_type.name == "Rectangle" => {
+                // hard-code the value for rectangle because many rectangle end up optimized away and we
+                // don't want to depend on the element.
+                Expression::Struct {
+                    ty: layout_info_type(),
+                    values: [("min", 0.), ("max", f32::MAX), ("preferred", 0.)]
                         .iter()
-                        .map(|(s, v)| (s.to_string(), Expression::NumberLiteral(*v, Unit::None))),
-                )
-                .collect(),
-        }
-    } else {
-        Expression::FunctionCall {
-            function: Box::new(Expression::BuiltinFunctionReference(
-                BuiltinFunction::ImplicitLayoutInfo(orientation),
-                None,
-            )),
-            arguments: vec![Expression::ElementReference(Rc::downgrade(elem))],
-            source_location: None,
-        }
+                        .map(|(s, v)| (s.to_string(), Expression::NumberLiteral(*v as _, Unit::Px)))
+                        .chain(
+                            [("min_percent", 0.), ("max_percent", 100.), ("stretch", 1.)]
+                                .iter()
+                                .map(|(s, v)| {
+                                    (s.to_string(), Expression::NumberLiteral(*v, Unit::None))
+                                }),
+                        )
+                        .collect(),
+                }
+            }
+            _ => Expression::FunctionCall {
+                function: Box::new(Expression::BuiltinFunctionReference(
+                    BuiltinFunction::ImplicitLayoutInfo(orientation),
+                    None,
+                )),
+                arguments: vec![Expression::ElementReference(Rc::downgrade(elem))],
+                source_location: None,
+            },
+        };
     }
 }
