@@ -72,7 +72,7 @@ pub fn lex_comment(text: &str, _: &mut LexState) -> usize {
                 if star > offset && bytes[star - 1] == b'/' {
                     nested += 1;
                     offset = star + 1;
-                } else if bytes[star + 1] == b'/' {
+                } else if star < bytes.len() - 1 && bytes[star + 1] == b'/' {
                     if nested == 0 {
                         return star + 2;
                     }
@@ -111,6 +111,7 @@ pub fn lex_string(text: &str, state: &mut LexState) -> usize {
     } else if !text.starts_with('"') {
         return 0;
     }
+    let text_len = text.as_bytes().len();
     let mut end = 1; // skip the '"'
     loop {
         let stop = match text[end..].find(&['"', '\\'][..]) {
@@ -123,11 +124,15 @@ pub fn lex_string(text: &str, state: &mut LexState) -> usize {
                 return stop + 1;
             }
             b'\\' => {
+                if text_len <= stop + 1 {
+                    // FIXME: report an error for unterminated string
+                    return 0;
+                }
                 if text.as_bytes()[stop + 1] == b'{' {
                     state.template_string_stack.push(0);
                     return stop + 2;
                 }
-                end = stop + 2;
+                end = stop + 1 + text[stop + 1..].chars().next().map_or(0, |c| c.len_utf8())
             }
             _ => unreachable!(),
         }
@@ -310,4 +315,16 @@ fn basic_lexer_test() {
             (crate::parser::SyntaxKind::Identifier, "j"),
         ],
     );
+
+    // Fuzzer tests:
+    compare(
+        r#"/**"#,
+        &[
+            (crate::parser::SyntaxKind::Div, "/"),
+            (crate::parser::SyntaxKind::Star, "*"),
+            (crate::parser::SyntaxKind::Star, "*"),
+        ],
+    );
+    compare(r#""\"#, &[(crate::parser::SyntaxKind::Error, "\"\\")]);
+    compare(r#""\ޱ"#, &[(crate::parser::SyntaxKind::Error, "\"\\ޱ")]);
 }
