@@ -23,10 +23,9 @@ use crate::parser;
 use crate::parser::{syntax_nodes, SyntaxKind, SyntaxNode};
 use crate::typeloader::ImportedTypes;
 use crate::typeregister::TypeRegister;
-use std::cell::RefCell;
+use std::cell::{Cell, RefCell};
 use std::collections::btree_map::Entry;
 use std::collections::{BTreeMap, HashMap};
-use std::iter::FromIterator;
 use std::rc::{Rc, Weak};
 
 macro_rules! unwrap_or_continue {
@@ -142,7 +141,7 @@ impl Document {
                     Some(import.file)
                 } else {
                     diag.push_error(
-                        format!("Unsupported foreign import {}", import.file),
+                        format!("Unsupported foreign import \"{}\"", import.file),
                         &import.import_token,
                     );
                     None
@@ -171,6 +170,7 @@ pub struct PopupWindow {
     pub component: Rc<Component>,
     pub x: NamedReference,
     pub y: NamedReference,
+    pub parent_element: ElementRc,
 }
 
 type ChildrenInsertionPoint = (ElementRc, syntax_nodes::ChildrenPlaceholder);
@@ -203,9 +203,9 @@ pub struct Component {
     pub optimized_elements: RefCell<Vec<ElementRc>>,
 
     /// Map of resources that should be embedded in the generated code, indexed by their absolute path on
-    /// disk on the build system and valued by a unique integer id, that can be used by the
-    /// generator for symbol generation.
-    pub embedded_file_resources: RefCell<HashMap<String, usize>>,
+    /// disk on the build system
+    pub embedded_file_resources:
+        RefCell<HashMap<String, crate::embedded_resources::EmbeddedResources>>,
 
     /// The layout constraints of the root item
     pub root_constraints: RefCell<LayoutConstraints>,
@@ -225,6 +225,10 @@ pub struct Component {
     /// The names under which this component should be accessible
     /// if it is a global singleton and exported.
     pub exported_global_names: RefCell<Vec<ExportedName>>,
+
+    /// This is the main entry point for the code generators. Such a component
+    /// should have the full API, etc.
+    pub is_root_component: Cell<bool>,
 }
 
 impl Component {
@@ -274,7 +278,7 @@ impl Component {
         if self.is_global() {
             !self.exported_global_names.borrow().is_empty()
         } else {
-            self.parent_element.upgrade().is_none()
+            self.parent_element.upgrade().is_none() && self.is_root_component.get()
         }
     }
 
@@ -288,6 +292,26 @@ impl Component {
             .map(|name| name.original_name())
             .collect()
     }
+
+    pub fn is_sub_component(&self) -> bool {
+        !self.is_root_component.get()
+            && self.parent_element.upgrade().is_none()
+            && !self.is_global()
+    }
+
+    // Number of repeaters in this component, including sub-components
+    pub fn repeater_count(&self) -> u32 {
+        let mut count = 0;
+        recurse_elem(&self.root_element, &(), &mut |element, _| {
+            let element = element.borrow();
+            if let Some(sub_component) = element.sub_component() {
+                count += sub_component.repeater_count();
+            } else if element.repeated.is_some() {
+                count += 1;
+            }
+        });
+        count
+    }
 }
 
 #[derive(Clone, Debug, Default)]
@@ -386,67 +410,7 @@ impl Clone for PropertyAnimation {
     }
 }
 
-// Wrapper around BindingsMap to offer an animation-preserving "insert" variant.
-#[derive(Debug, Clone, Default, derive_more::Deref, derive_more::DerefMut)]
-pub struct BindingsMap(BTreeMap<String, BindingExpression>);
-
-impl BindingsMap {
-    pub fn set_binding_if_not_set(
-        &mut self,
-        property_name: String,
-        expression_fn: impl FnOnce() -> Expression,
-    ) {
-        match self.0.entry(property_name) {
-            Entry::Vacant(vacant_entry) => {
-                let mut binding: BindingExpression = expression_fn().into();
-                binding.priority = i32::MAX;
-                vacant_entry.insert(binding);
-            }
-            Entry::Occupied(mut existing_entry) if !existing_entry.get().has_binding() => {
-                let mut binding: BindingExpression = expression_fn().into();
-                binding.priority = i32::MAX;
-                existing_entry.get_mut().merge_with(&binding);
-            }
-            _ => {}
-        };
-    }
-}
-
-impl IntoIterator for BindingsMap {
-    type Item = (String, BindingExpression);
-
-    type IntoIter = std::collections::btree_map::IntoIter<String, BindingExpression>;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.0.into_iter()
-    }
-}
-
-impl<'a> IntoIterator for &'a BindingsMap {
-    type Item = (&'a String, &'a BindingExpression);
-
-    type IntoIter = std::collections::btree_map::Iter<'a, String, BindingExpression>;
-
-    fn into_iter(self) -> Self::IntoIter {
-        (&self.0).iter()
-    }
-}
-
-impl<'a> IntoIterator for &'a mut BindingsMap {
-    type Item = (&'a String, &'a mut BindingExpression);
-
-    type IntoIter = std::collections::btree_map::IterMut<'a, String, BindingExpression>;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.0.iter_mut()
-    }
-}
-
-impl FromIterator<(String, BindingExpression)> for BindingsMap {
-    fn from_iter<T: IntoIterator<Item = (String, BindingExpression)>>(iter: T) -> Self {
-        Self(BTreeMap::from_iter(iter))
-    }
-}
+pub type BindingsMap = BTreeMap<String, RefCell<BindingExpression>>;
 
 /// An Element is an instantiation of a Component
 #[derive(Default)]
@@ -538,10 +502,11 @@ pub fn pretty_print(
         }
     }
     for (name, expr) in &e.bindings {
+        let expr = expr.borrow();
         indent!();
         write!(f, "{}: ", name)?;
         expression_tree::pretty_print(f, &expr.expression)?;
-        if expr.analysis.borrow().as_ref().map_or(false, |a| a.is_const) {
+        if expr.analysis.as_ref().map_or(false, |a| a.is_const) {
             write!(f, "/*const*/")?;
         }
         writeln!(f, ";")?;
@@ -581,15 +546,38 @@ pub struct PropertyAnalysis {
     /// true if somewhere in the code, there is an expression that changes this property with an assignment
     pub is_set: bool,
 
+    /// True if this property might be set from a different component.
+    pub is_set_externally: bool,
+
     /// true if somewhere in the code, an expression is reading this property
     /// Note: currently this is only set in the binding analysis pass
     pub is_read: bool,
+
+    /// true if this property is read from another component
+    pub is_read_externally: bool,
 }
 
 impl PropertyAnalysis {
-    /// Merge analysis from base element for e.g. inlining
+    /// Merge analysis from base element for inlining
+    ///
+    /// Contrary to `merge`, we don't keep the external uses because
+    /// they should come from us
+    pub fn merge_with_base(&mut self, other: &PropertyAnalysis) {
+        self.is_set |= other.is_set;
+        self.is_read |= other.is_read;
+    }
+
+    /// Merge the analysis
     pub fn merge(&mut self, other: &PropertyAnalysis) {
         self.is_set |= other.is_set;
+        self.is_read |= other.is_read;
+        self.is_read_externally |= other.is_read_externally;
+        self.is_set_externally |= other.is_set_externally;
+    }
+
+    /// Return true if it is read or set or used in any way
+    pub fn is_used(&self) -> bool {
+        self.is_read || self.is_read_externally || self.is_set || self.is_set_externally
     }
 }
 
@@ -717,7 +705,10 @@ impl Element {
 
             if let Some(csn) = prop_decl.BindingExpression() {
                 if r.bindings
-                    .insert(prop_name.to_string(), BindingExpression::new_uncompiled(csn.into()))
+                    .insert(
+                        prop_name.to_string(),
+                        BindingExpression::new_uncompiled(csn.into()).into(),
+                    )
                     .is_some()
                 {
                     diag.push_error(
@@ -728,7 +719,7 @@ impl Element {
             }
             if let Some(csn) = prop_decl.TwoWayBinding() {
                 if r.bindings
-                    .insert(prop_name.into(), BindingExpression::new_uncompiled(csn.into()))
+                    .insert(prop_name.into(), BindingExpression::new_uncompiled(csn.into()).into())
                     .is_some()
                 {
                     diag.push_error(
@@ -754,7 +745,9 @@ impl Element {
         if let Type::Builtin(builtin_base) = &r.base_type {
             for (prop, info) in &builtin_base.properties {
                 if let Some(expr) = &info.default_value {
-                    r.bindings.entry(prop.clone()).or_insert_with(|| expr.clone().into());
+                    r.bindings
+                        .entry(prop.clone())
+                        .or_insert_with(|| RefCell::new(expr.clone().into()));
                 }
             }
         }
@@ -764,7 +757,8 @@ impl Element {
                 unwrap_or_continue!(parser::identifier_text(&sig_decl.DeclaredIdentifier()); diag);
 
             if let Some(csn) = sig_decl.TwoWayBinding() {
-                r.bindings.insert(name.clone(), BindingExpression::new_uncompiled(csn.into()));
+                r.bindings
+                    .insert(name.clone(), BindingExpression::new_uncompiled(csn.into()).into());
                 r.property_declarations.insert(
                     name,
                     PropertyDeclaration {
@@ -819,7 +813,7 @@ impl Element {
             if r.bindings
                 .insert(
                     resolved_name.into_owned(),
-                    BindingExpression::new_uncompiled(con_node.clone().into()),
+                    BindingExpression::new_uncompiled(con_node.clone().into()).into(),
                 )
                 .is_some()
             {
@@ -862,9 +856,10 @@ impl Element {
                                     let mut r = BindingExpression::from(Expression::Invalid);
                                     r.priority = 1;
                                     r.span = Some(prop_name_token.to_source_location());
-                                    r
+                                    r.into()
                                 });
                             if expr_binding
+                                .get_mut()
                                 .animation
                                 .replace(PropertyAnimation::Static(anim_element))
                                 .is_some()
@@ -1128,7 +1123,7 @@ impl Element {
 
             if self
                 .bindings
-                .insert(resolved_name.to_string(), BindingExpression::new_uncompiled(b))
+                .insert(resolved_name.to_string(), BindingExpression::new_uncompiled(b).into())
                 .is_some()
             {
                 diag.push_error("Duplicated property binding".into(), &name_token);
@@ -1178,6 +1173,57 @@ impl Element {
             .map(|n| n.to_string())
             .unwrap_or_else(|| self.id.clone())
     }
+
+    /// Return true if the binding is set, either on this element or in a base
+    ///
+    /// If `need_explicit` is true, then only consider binding set in the code, not the ones set
+    /// by the compiler later.
+    pub fn is_binding_set(self: &Element, property_name: &str, need_explicit: bool) -> bool {
+        if self.bindings.get(property_name).map_or(false, |b| {
+            b.borrow().has_binding() && (!need_explicit || b.borrow().priority > 0)
+        }) {
+            true
+        } else if let Type::Component(base) = &self.base_type {
+            base.root_element.borrow().is_binding_set(property_name, need_explicit)
+        } else {
+            false
+        }
+    }
+
+    /// Set the property `property_name` of this Element only if it was not set.
+    /// the `expression_fn` will only be called if it isn't set
+    pub fn set_binding_if_not_set(
+        &mut self,
+        property_name: String,
+        expression_fn: impl FnOnce() -> Expression,
+    ) {
+        if self.is_binding_set(&property_name, false) {
+            return;
+        }
+
+        match self.bindings.entry(property_name) {
+            Entry::Vacant(vacant_entry) => {
+                let mut binding: BindingExpression = expression_fn().into();
+                binding.priority = i32::MAX;
+                vacant_entry.insert(binding.into());
+            }
+            Entry::Occupied(mut existing_entry) => {
+                let mut binding: BindingExpression = expression_fn().into();
+                binding.priority = i32::MAX;
+                existing_entry.get_mut().get_mut().merge_with(&binding);
+            }
+        };
+    }
+
+    pub fn sub_component(&self) -> Option<&Rc<Component>> {
+        if self.repeated.is_some() {
+            None
+        } else if let Type::Component(sub_component) = &self.base_type {
+            Some(sub_component)
+        } else {
+            None
+        }
+    }
 }
 
 /// Create a Type for this node
@@ -1384,7 +1430,9 @@ pub fn recurse_elem_including_sub_components<State>(
         ));
         if elem.borrow().repeated.is_some() {
             if let Type::Component(base) = &elem.borrow().base_type {
-                recurse_elem_including_sub_components(base, state, vis);
+                if base.parent_element.upgrade().is_some() {
+                    recurse_elem_including_sub_components(base, state, vis);
+                }
             }
         }
         vis(elem, state)
@@ -1434,7 +1482,13 @@ pub fn recurse_elem_including_sub_components_no_borrow<State>(
         .popup_windows
         .borrow()
         .iter()
-        .for_each(|p| recurse_elem_including_sub_components_no_borrow(&p.component, state, vis))
+        .for_each(|p| recurse_elem_including_sub_components_no_borrow(&p.component, state, vis));
+    component
+        .used_types
+        .borrow()
+        .globals
+        .iter()
+        .for_each(|p| recurse_elem_including_sub_components_no_borrow(&p, state, vis));
 }
 
 /// This visit the binding attached to this element, but does not recurse in children elements
@@ -1450,11 +1504,12 @@ pub fn visit_element_expressions(
         elem: &ElementRc,
         vis: &mut impl FnMut(&mut Expression, Option<&str>, &dyn Fn() -> Type),
     ) {
-        let mut bindings = std::mem::take(&mut elem.borrow_mut().bindings);
-        for (name, expr) in &mut bindings {
-            vis(expr, Some(name.as_str()), &|| elem.borrow().lookup_property(name).property_type);
+        for (name, expr) in &elem.borrow().bindings {
+            vis(&mut *expr.borrow_mut(), Some(name.as_str()), &|| {
+                elem.borrow().lookup_property(name).property_type
+            });
 
-            match &mut expr.animation {
+            match &mut expr.borrow_mut().animation {
                 Some(PropertyAnimation::Static(e)) => visit_element_expressions_simple(e, vis),
                 Some(PropertyAnimation::Transition { animations, state_ref }) => {
                     vis(state_ref, None, &|| Type::Int32);
@@ -1465,7 +1520,6 @@ pub fn visit_element_expressions(
                 None => (),
             }
         }
-        elem.borrow_mut().bindings = bindings;
     }
 
     let repeated = std::mem::take(&mut elem.borrow_mut().repeated);
@@ -1554,13 +1608,11 @@ pub fn visit_all_named_references_in_element(
     elem.borrow_mut().layout_info_prop = layout_info_prop;
 
     // visit two way bindings
-    let mut bindings = std::mem::take(&mut elem.borrow_mut().bindings);
-    for (_, expr) in &mut bindings {
-        for nr in &mut expr.two_way_bindings {
+    for (_, expr) in &elem.borrow().bindings {
+        for nr in &mut expr.borrow_mut().two_way_bindings {
             vis(nr);
         }
     }
-    elem.borrow_mut().bindings = bindings;
 
     let mut property_declarations = std::mem::take(&mut elem.borrow_mut().property_declarations);
     for pd in property_declarations.values_mut() {
