@@ -25,8 +25,9 @@ pub fn check_expressions(doc: &crate::object_tree::Document, diag: &mut BuildDia
 
 fn check_expression(component: &Rc<Component>, e: &Expression, diag: &mut BuildDiagnostics) {
     match e {
-        Expression::MemberFunction { base_node, .. } => {
-            diag.push_error("Member function must be called".into(), base_node);
+        Expression::MemberFunction { .. } => {
+            // Must already have been be reported.
+            debug_assert!(diag.has_error());
         }
         Expression::BuiltinMacroReference(_, node) => {
             diag.push_error("Builtin function must be called".into(), node);
