@@ -16,6 +16,7 @@ LICENSE END */
     This pass must be run after lower_layout
 */
 
+use std::cell::RefCell;
 use std::rc::Rc;
 
 use crate::diagnostics::BuildDiagnostics;
@@ -35,8 +36,8 @@ pub fn default_geometry(root_component: &Rc<Component>, diag: &mut BuildDiagnost
 
             gen_layout_info_prop(elem);
 
-            let base_type = elem.borrow().base_type.clone();
-            if let (Some(parent), Type::Builtin(builtin_type)) = (parent, base_type) {
+            let base_type = elem.borrow().builtin_type();
+            if let (Some(parent), Some(builtin_type)) = (parent, base_type) {
                 match builtin_type.default_size_binding {
                     DefaultSizeBinding::None => {}
                     DefaultSizeBinding::ExpandsToParentGeometry => {
@@ -53,16 +54,13 @@ pub fn default_geometry(root_component: &Rc<Component>, diag: &mut BuildDiagnost
                                 property_type == Type::LogicalLength
                             });
 
-                            elem.borrow_mut()
-                                .bindings
-                                .get(property)
-                                .map_or(false, |b| b.priority > 0)
+                            elem.borrow().is_binding_set(property, true)
                         };
 
                         let width_specified = has_length_property_binding(elem, "width");
                         let height_specified = has_length_property_binding(elem, "height");
 
-                        let is_image = matches!(elem.borrow().builtin_type(), Some(builtin) if builtin.name == "Image");
+                        let is_image = builtin_type.name == "Image";
 
                         if !elem.borrow().child_of_layout {
                             // Add aspect-ratio preserving width or height bindings
@@ -87,7 +85,7 @@ pub fn default_geometry(root_component: &Rc<Component>, diag: &mut BuildDiagnost
                                     property_type: image_fit_prop_type,
                                 } = elem.borrow().lookup_property("image-fit");
 
-                                elem.borrow_mut().bindings.set_binding_if_not_set(
+                                elem.borrow_mut().set_binding_if_not_set(
                                     image_fit_prop_name.into(),
                                     || {
                                         Expression::EnumerationValue(
@@ -160,9 +158,9 @@ fn gen_layout_info_prop(elem: &ElementRc) {
     }
 
     let expr_v = BindingExpression::new_with_span(expr_v, elem.borrow().to_source_location());
-    li_v.element().borrow_mut().bindings.insert(li_v.name().into(), expr_v);
+    li_v.element().borrow_mut().bindings.insert(li_v.name().into(), expr_v.into());
     let expr_h = BindingExpression::new_with_span(expr_h, elem.borrow().to_source_location());
-    li_h.element().borrow_mut().bindings.insert(li_h.name().into(), expr_h);
+    li_h.element().borrow_mut().bindings.insert(li_h.name().into(), expr_h.into());
 }
 
 /// Replace expression such as  `"width: 30%;` with `width: 0.3 * parent.width;`
@@ -172,11 +170,16 @@ fn fix_percent_size(
     property: &str,
     diag: &mut BuildDiagnostics,
 ) {
-    if !elem.borrow().bindings.get(property).map_or(false, |b| b.ty() == Type::Percent) {
+    let elem = elem.borrow();
+    let binding = match elem.bindings.get(property) {
+        Some(b) => b,
+        None => return,
+    };
+
+    if binding.borrow().ty() != Type::Percent {
         return;
     }
-    let mut elem = elem.borrow_mut();
-    let b = elem.bindings.get_mut(property).unwrap();
+    let mut b = binding.borrow_mut();
     if let Some(parent) = parent {
         debug_assert_eq!(
             parent.borrow().lookup_property(property),
@@ -205,15 +208,16 @@ fn make_default_100(elem: &ElementRc, parent_element: &ElementRc, property: &str
     if property_type != Type::LogicalLength {
         return;
     }
-    elem.borrow_mut().bindings.set_binding_if_not_set(resolved_name.to_string(), || {
+    elem.borrow_mut().set_binding_if_not_set(resolved_name.to_string(), || {
         Expression::PropertyReference(NamedReference::new(parent_element, resolved_name.as_ref()))
     });
 }
 
 fn make_default_implicit(elem: &ElementRc, property: &str, orientation: Orientation) {
     let base = crate::layout::implicit_layout_info_call(elem, orientation).into();
-    elem.borrow_mut().bindings.set_binding_if_not_set(property.into(), || {
-        Expression::StructFieldAccess { base, name: "preferred".into() }
+    elem.borrow_mut().set_binding_if_not_set(property.into(), || Expression::StructFieldAccess {
+        base,
+        name: "preferred".into(),
     });
 }
 
@@ -229,7 +233,7 @@ fn make_default_aspect_ratio_preserving_binding(
     missing_size_property: &str,
     given_size_property: &str,
 ) {
-    if elem.borrow().bindings.contains_key(missing_size_property) {
+    if elem.borrow().is_binding_set(missing_size_property, false) {
         return;
     }
 
@@ -273,5 +277,7 @@ fn make_default_aspect_ratio_preserving_binding(
         },
     ]);
 
-    elem.borrow_mut().bindings.insert(missing_size_property.to_string(), binding.into());
+    elem.borrow_mut()
+        .bindings
+        .insert(missing_size_property.to_string(), RefCell::new(binding.into()));
 }
