@@ -18,8 +18,9 @@ use std::collections::HashSet;
 use std::rc::Rc;
 
 pub fn ensure_window(component: &Rc<Component>, type_register: &TypeRegister) {
-    if matches!(component.root_element.borrow().base_type.to_string().as_str(), "Window" | "Dialog")
-    {
+    if component.root_element.borrow().builtin_type().map_or(true, |b| {
+        matches!(b.name.as_str(), "Window" | "Dialog" | "WindowItem" | "PopupWindow")
+    }) {
         return; // already a window, nothing to do
     }
 
@@ -54,7 +55,9 @@ pub fn ensure_window(component: &Rc<Component>, type_register: &TypeRegister) {
     let make_two_way = |name: &str| {
         new_root.borrow_mut().bindings.insert(
             name.into(),
-            BindingExpression::new_two_way(NamedReference::new(&win_elem, name)),
+            RefCell::new(BindingExpression::new_two_way(
+                NamedReference::new(&win_elem, name).into(),
+            )),
         );
     };
     make_two_way("width");
