@@ -27,6 +27,10 @@ use crate::langtype::{NativeClass, Type};
 use crate::object_tree::{Component, Element, ElementRc};
 use crate::typeregister::TypeRegister;
 
+pub fn is_flickable_element(element: &ElementRc) -> bool {
+    matches!(&element.borrow().base_type, Type::Builtin(n) if n.name == "Flickable")
+}
+
 pub fn handle_flickable(root_component: &Rc<Component>, tr: &TypeRegister) {
     let mut native_rect = tr.lookup("Rectangle").as_builtin().native_class.clone();
     while let Some(p) = native_rect.parent.clone() {
@@ -37,7 +41,7 @@ pub fn handle_flickable(root_component: &Rc<Component>, tr: &TypeRegister) {
         root_component,
         &(),
         &mut |elem: &ElementRc, _| {
-            if !matches!(&elem.borrow().base_type, Type::Builtin(n) if n.name == "Flickable") {
+            if !is_flickable_element(elem) {
                 return;
             }
 
@@ -64,10 +68,10 @@ fn create_viewport_element(flickable_elem: &ElementRc, native_rect: &Rc<NativeCl
             flickable.property_declarations.insert(prop.to_owned(), info.ty.clone().into());
             match flickable.bindings.entry(prop.to_owned()) {
                 std::collections::btree_map::Entry::Occupied(entry) => {
-                    entry.into_mut().two_way_bindings.push(nr);
+                    entry.into_mut().get_mut().two_way_bindings.push(nr);
                 }
                 std::collections::btree_map::Entry::Vacant(entry) => {
-                    entry.insert(BindingExpression::new_two_way(nr));
+                    entry.insert(BindingExpression::new_two_way(nr).into());
                 }
             }
         }
@@ -147,14 +151,15 @@ fn is_layout(base_type: &Type) -> bool {
 
 /// Set the property binding on the given element to the given expression (computed lazily).
 /// The parameter to the lazily calculation is the element's children
-pub fn set_binding_if_not_explicit(
+fn set_binding_if_not_explicit(
     elem: &ElementRc,
     property: &str,
     expression: impl FnOnce() -> Option<Expression>,
 ) {
-    if !elem.borrow().bindings.contains_key(property) {
+    // we can't use `set_binding_if_not_set` directly because `expression()` may borrow `elem`
+    if elem.borrow().bindings.get(property).map_or(true, |b| !b.borrow().has_binding()) {
         if let Some(e) = expression() {
-            elem.borrow_mut().bindings.insert(property.into(), e.into());
+            elem.borrow_mut().set_binding_if_not_set(property.into(), || e);
         }
     }
 }
