@@ -71,10 +71,10 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
     let mut property_sets = PropertySets::default();
     recurse_elem_including_sub_components(component, &(), &mut |e, _| {
         'bindings: for (name, binding) in &e.borrow().bindings {
-            for nr in &binding.two_way_bindings {
+            for nr in &binding.borrow().two_way_bindings {
                 let other_e = nr.element();
                 if name == nr.name() && Rc::ptr_eq(e, &other_e) {
-                    diag.push_error("Property cannot alias to itself".into(), binding);
+                    diag.push_error("Property cannot alias to itself".into(), &*binding.borrow());
                     continue 'bindings;
                 }
                 property_sets.add_link(NamedReference::new(e, name), nr.clone());
@@ -115,22 +115,23 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
 
         // adjust the bindings
         let old_binding = elem.borrow_mut().bindings.remove(remove.name());
-        let must_simplify = if let Some(mut binding) = old_binding {
+        let must_simplify = if let Some(mut binding) = old_binding.map(RefCell::into_inner) {
             remove_from_binding_expression(&mut binding, &to);
             if binding.has_binding() {
                 let to_elem = to.element();
                 match to_elem.borrow_mut().bindings.entry(to.name().to_owned()) {
                     Entry::Occupied(mut e) => {
-                        remove_from_binding_expression(e.get_mut(), &to);
-                        if e.get().priority < binding.priority || !e.get().has_binding() {
-                            e.get_mut().merge_with(&binding);
+                        let b = e.get_mut().get_mut();
+                        remove_from_binding_expression(b, &to);
+                        if b.priority < binding.priority || !b.has_binding() {
+                            b.merge_with(&binding);
                         } else {
-                            binding.merge_with(e.get());
-                            *e.get_mut() = binding;
+                            binding.merge_with(b);
+                            *b = binding;
                         }
                     }
                     Entry::Vacant(e) => {
-                        e.insert(binding);
+                        e.insert(binding.into());
                     }
                 };
                 false
@@ -145,8 +146,8 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
             let to_elem = to.element();
             let mut to_elem = to_elem.borrow_mut();
             if let Some(b) = to_elem.bindings.get_mut(to.name()) {
-                remove_from_binding_expression(b, &to);
-                if !b.has_binding() {
+                remove_from_binding_expression(b.get_mut(), &to);
+                if !b.get_mut().has_binding() {
                     to_elem.bindings.remove(to.name());
                 }
             }
@@ -155,24 +156,35 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
         // Remove the declaration
         {
             let mut elem = elem.borrow_mut();
+            let used_externally = elem
+                .property_analysis
+                .borrow()
+                .get(remove.name())
+                .map_or(false, |v| v.is_read_externally || v.is_set_externally);
             if let Some(d) = elem.property_declarations.get_mut(remove.name()) {
-                if d.expose_in_public_api {
+                if d.expose_in_public_api || used_externally {
                     d.is_alias = Some(to.clone());
                     drop(elem);
                     // one must mark the aliased property as settable from outside
-                    to.element()
-                        .borrow()
-                        .property_analysis
-                        .borrow_mut()
-                        .entry(to.name().into())
-                        .or_default()
-                        .is_set = true;
+                    to.mark_as_set();
                 } else {
                     elem.property_declarations.remove(remove.name());
+                    let analysis = elem.property_analysis.borrow().get(remove.name()).cloned();
+                    if let Some(analysis) = analysis {
+                        drop(elem);
+                        to.element()
+                            .borrow()
+                            .property_analysis
+                            .borrow_mut()
+                            .entry(to.name().to_owned())
+                            .or_default()
+                            .merge(&analysis);
+                    };
                 }
             } else {
                 // This is not a declaration, we must re-create the binding
-                elem.bindings.insert(remove.name().to_owned(), BindingExpression::new_two_way(to));
+                elem.bindings
+                    .insert(remove.name().to_owned(), BindingExpression::new_two_way(to).into());
             }
         }
     }
@@ -192,6 +204,7 @@ fn best_property(
     macro_rules! canonical_order {
         ($x: expr) => {{
             (
+                !$x.element().borrow().enclosing_component.upgrade().unwrap().is_global(),
                 is_declaration(&$x),
                 !Rc::ptr_eq(&component.root_element, &$x.element()),
                 $x.element().borrow().id.clone(),
