@@ -39,19 +39,18 @@ fn reorder_children_by_zorder(
             .borrow_mut()
             .bindings
             .remove("z")
-            .and_then(|e| eval_const_expr(&e.expression, "z", &e, diag));
-        let z = z.or_else(|| {
-            child_elm.borrow().repeated.as_ref()?;
-            if let Type::Component(c) = &child_elm.borrow().base_type {
-                c.root_element
-                    .borrow_mut()
-                    .bindings
-                    .remove("z")
-                    .and_then(|e| eval_const_expr(&e.expression, "z", &e, diag))
-            } else {
-                None
-            }
-        });
+            .and_then(|e| eval_const_expr(&e.borrow().expression, "z", &*e.borrow(), diag));
+        let z =
+            z.or_else(|| {
+                child_elm.borrow().repeated.as_ref()?;
+                if let Type::Component(c) = &child_elm.borrow().base_type {
+                    c.root_element.borrow_mut().bindings.remove("z").and_then(|e| {
+                        eval_const_expr(&e.borrow().expression, "z", &*e.borrow(), diag)
+                    })
+                } else {
+                    None
+                }
+            });
 
         if let Some(z) = z {
             if children_z_order.is_empty() {
